@@ -2,7 +2,7 @@ use futures::{pin_mut, stream::FuturesUnordered, Future, FutureExt, SinkExt, Str
 use futures_channel::mpsc::{UnboundedReceiver, UnboundedSender};
 use futures_util::select;
 use js_sys::Reflect;
-use log::{debug, warn};
+use log::{debug, error, warn};
 use serde::Serialize;
 use std::{collections::HashMap, pin::Pin};
 use uuid::Uuid;
@@ -122,14 +122,6 @@ async fn message_loop(
         .await
         .expect("failed to connect to signalling server");
 
-    // TODO: this is the ideal place to start the websocket connection, unfortunately
-    // *something* isn't quite right when this is polled through a bevy system and the
-    // connection is never established. So it's currently moved outside the messaging loop
-    // when/if I figure out what's wrong, it should ideally be moved back inside again.
-    // let (_ws, mut wsio) = WsMeta::connect(&room_url, None)
-    //     .await
-    //     .expect("failed to connect to signalling server");
-
     let (requests_sender, mut requests_receiver) =
         futures_channel::mpsc::unbounded::<PeerRequest>();
 
@@ -167,35 +159,38 @@ async fn message_loop(
             },
 
             message = next_signal_event => {
-                let message = match message.unwrap() {
-                    WsMessage::Text(message) => message,
-                    WsMessage::Binary(_) => panic!("binary data from signal server"),
+                match message {
+                    Some(WsMessage::Text(message)) => {
+                        debug!("{}", message);
+
+                        let event: PeerEvent = serde_json::from_str(&message)
+                            .expect(&format!("couldn't parse peer event {}", message));
+
+                        match event {
+                            PeerEvent::NewPeer(peer_uuid) => {
+                                let (signal_sender, signal_receiver) = futures_channel::mpsc::unbounded();
+                                handshake_signals.insert(peer_uuid.clone(), signal_sender);
+                                let signal_peer = SignalPeer::new(peer_uuid, requests_sender.clone());
+                                offer_handshakes.push(handshake_offer(signal_peer, signal_receiver, messages_from_peers_tx.clone()));
+                            }
+                            PeerEvent::Signal { sender, data } => {
+                                let from_peer_sender = handshake_signals.entry(sender.clone()).or_insert_with(|| {
+                                    let (from_peer_sender, from_peer_receiver) = futures_channel::mpsc::unbounded();
+                                    let signal_peer = SignalPeer::new(sender, requests_sender.clone());
+                                    // We didn't start signalling with this peer, assume we're the accepting part
+                                    accept_handshakes.push(handshake_accept(signal_peer, from_peer_receiver, messages_from_peers_tx.clone()));
+                                    from_peer_sender
+                                });
+                                from_peer_sender.unbounded_send(data)
+                                    .expect("failed to forward signal to handshaker");
+                            }
+                        }
+                    },
+                    Some(WsMessage::Binary(_)) => {
+                        error!("Received binary data from signal server (expected text). Ignoring.");
+                    },
+                    None => {} // Disconnected from signalling server
                 };
-
-                debug!("{}", message);
-
-                let event: PeerEvent = serde_json::from_str(&message)
-                    .expect(&format!("couldn't parse peer event {}", message));
-
-                match event {
-                    PeerEvent::NewPeer(peer_uuid) => {
-                        let (signal_sender, signal_receiver) = futures_channel::mpsc::unbounded();
-                        handshake_signals.insert(peer_uuid.clone(), signal_sender);
-                        let signal_peer = SignalPeer::new(peer_uuid, requests_sender.clone());
-                        offer_handshakes.push(handshake_offer(signal_peer, signal_receiver, messages_from_peers_tx.clone()));
-                    }
-                    PeerEvent::Signal { sender, data } => {
-                        let from_peer_sender = handshake_signals.entry(sender.clone()).or_insert_with(|| {
-                            let (from_peer_sender, from_peer_receiver) = futures_channel::mpsc::unbounded();
-                            let signal_peer = SignalPeer::new(sender, requests_sender.clone());
-                            // We didn't start signalling with this peer, assume we're the accepting part
-                            accept_handshakes.push(handshake_accept(signal_peer, from_peer_receiver, messages_from_peers_tx.clone()));
-                            from_peer_sender
-                        });
-                        from_peer_sender.unbounded_send(data)
-                            .expect("failed to forward signal to handshaker");
-                    }
-                }
             }
 
             request = next_request => {
@@ -277,15 +272,29 @@ async fn handshake_offer(
 
     signal_peer.send(PeerSignal::Offer(conn.local_description().unwrap().sdp()));
 
-    let signal = signal_receiver.next().await.ok_or("No more signals :(")?;
+    let sdp: String;
 
-    let answer = match signal {
-        PeerSignal::Answer(answer) => answer,
-        PeerSignal::Offer(_) => panic!("offers shouldn't get here... I think"),
-        PeerSignal::IceCandidate(_) => panic!("HELP ICE"),
-    };
+    loop {
+        let signal = signal_receiver
+            .next()
+            .await
+            .ok_or("Signal server connection lost in the middle of a handshake")?;
 
-    let sdp = answer;
+        match signal {
+            PeerSignal::Answer(answer) => {
+                sdp = answer;
+                break;
+            }
+            PeerSignal::Offer(_) => {
+                warn!("Got an unexpected Offer, while waiting for Answer. Ignoring.")
+            }
+            PeerSignal::IceCandidate(_) => {
+                warn!(
+                    "Got an ice candidate message, but ice trickle is not yet supported. Ignoring."
+                )
+            }
+        };
+    }
 
     let mut remote_description: RtcSessionDescriptionInit =
         RtcSessionDescriptionInit::new(RtcSdpType::Answer);
